import React from "react";

import AWS from 'aws-sdk/global';
import Kendra from 'aws-sdk/clients/kendra';

import { QueryResultType } from "./constants";
import { isNullOrUndefined } from "./utils";
import { Spinner } from "react-bootstrap";
import {
  QueryResult,
  QueryResultItem,
  QueryResultItemList
} from "./kendraTypes";

// import { getSearchResults } from "./exampleData/getSearchResults";

import SearchBar from "./searchBar/SearchBar";
import ResultPanel from "./resultsPanel/ResultPanel";
import Pagination from "./pagination/Pagination";

import "bootstrap/dist/css/bootstrap.min.css";
import "./search.scss";

interface SearchProps { }

interface SearchState {
  dataReady: boolean;
  searchResults: QueryResult;
  topResults: QueryResultItemList;
  faqResults: QueryResultItemList;
  docResults: QueryResultItemList;
  currentPageNumber: number;
  queryText: string;
}

export default class Search extends React.Component<SearchProps, SearchState> {
  constructor(props: SearchProps) {
    super(props);

    this.state = {
      dataReady: false,
      searchResults: {},
      topResults: [],
      faqResults: [],
      docResults: [],
      currentPageNumber: 1,
      queryText: ""
    };
  }

  getResults = async (queryText: string, pageNumber: number = 1) => {
    this.setState({ dataReady: false });

    /*
      replace the line below with the api call
      Kendra.query({ IndexId: indexId, QueryText: queryText, PageNumber: pageNumber})    
    */
    // const staticResults = getSearchResults(pageNumber);
    // const results = Kendra.query({ IndexId: process.env.REACT_APP_NOT_SECRET_CODE, QueryText: queryText, PageNumber: pageNumber})

    var credentials = new AWS.Credentials({
      accessKeyId: process.env.REACT_APP_AWS_ACCESS_KEY_ID!,
      secretAccessKey: process.env.REACT_APP_AWS_SECRET_ACCESS_KEY!,
      sessionToken: process.env.REACT_APP_AWS_SESSION_TOKEN!
    });

    const params = {
      IndexId: process.env.REACT_APP_INDEX!,
      QueryText: queryText,
      PageNumber: pageNumber
    };

    var kendra = new Kendra({
      apiVersion: '2019-02-03',
      region: process.env.REACT_APP_AWS_DEFAULT_REGION,
      credentials
    });

    const results = await kendra.query(params).promise();

    const tempTopResults: QueryResultItemList = [];
    const tempFAQResults: QueryResultItemList = [];
    const tempDocumentResults: QueryResultItemList = [];
    if (results && results.ResultItems) {
      results.ResultItems.forEach((result: QueryResultItem) => {
        switch (result.Type) {
          case QueryResultType.Answer:
            tempTopResults.push(result);
            break;
          case QueryResultType.QuestionAnswer:
            tempFAQResults.push(result);
            break;
          case QueryResultType.Document:
            tempDocumentResults.push(result);
            break;
          default:
            break;
        }
      });
      this.setState({
        searchResults: results,
        topResults: tempTopResults,
        faqResults: tempFAQResults,
        docResults: tempDocumentResults,
        dataReady: true
      });
    } else {
      this.setState({
        searchResults: {},
        topResults: tempTopResults,
        faqResults: tempFAQResults,
        docResults: tempDocumentResults,
        dataReady: true
      });
    }
    this.setState({
      currentPageNumber: pageNumber,
      queryText: queryText
    });
  };

  render() {
    return (
      <div>
        <SearchBar onSubmit={this.getResults} />
        {this.state.queryText &&
          this.state.dataReady &&
          !isNullOrUndefined(this.state.searchResults) && (
            <div>
              <ResultPanel
                results={this.state.searchResults}
                topResults={this.state.topResults}
                faqResults={this.state.faqResults}
                docResults={this.state.docResults}
                dataReady={this.state.dataReady}
                currentPageNumber={this.state.currentPageNumber}
              />
              <Pagination
                queryText={this.state.queryText}
                currentPageNumber={this.state.currentPageNumber}
                onSubmit={this.getResults}
                results={this.state.searchResults}
              />
            </div>
          )}

        {this.state.queryText && !this.state.dataReady && (
          <div className="results-section center-align">
            <Spinner animation="border" variant="secondary" />
          </div>
        )}
        {this.state.dataReady &&
          this.state.searchResults.TotalNumberOfResults === 0 && (
            <div className="results-section empty-results">
              Kendra didn't match any results to your query.
            </div>
          )}
      </div>
    );
  }
}
